package main

import (
	"embed"
	"fmt"
	"github.com/gin-gonic/gin"
	log "github.com/sirupsen/logrus"
	"github.com/spf13/viper"
	"io/fs"
	"net"
	"net/http"
	"os"
	"strconv"
	"syscall"
)

//go:embed assets
var assets embed.FS

var (
	router   *gin.Engine
	listener net.Listener
)

func webSetup() {
	// Set log level according to
	if log.GetLevel() == log.DebugLevel {
		gin.SetMode(gin.DebugMode)
	} else {
		gin.SetMode(gin.ReleaseMode)
	}

	// Configure gin router
	router = gin.New()
	router.Use(recovery())
	router.ForwardedByClientIP = viper.GetStringMap("server")["proxy"].(bool)

	// Configure logger
	if log.GetLevel() == log.DebugLevel {
		router.Use(gin.LoggerWithWriter(logger{}))
	}

	// Redirect on no route
	router.NoRoute(func(context *gin.Context) {
		context.Redirect(http.StatusTemporaryRedirect, "/web")
	})
}

func listenerSetup() {
	var err error
	switch serverConfig["unix"].(bool) {
	case false:
		address := fmt.Sprintf("%s:%s",
			serverConfig["host"].(string),
			strconv.Itoa(int(serverConfig["port"].(int64))))
		listener, err = net.Listen("tcp", address)
		if err != nil {
			log.Errorf("Error binding %s, %s", address, err)
		}
		log.Infof("Web server listening on %s.", address)
	case true:
		path := serverConfig["host"].(string)
		listener, err = net.Listen("unix", path)
		if err != nil {
			log.Errorf("Error binding %s, %s", path, err)
		}

		err = syscall.Chmod(path, 0777)
		if err != nil {
			log.Errorf("Error changing permission of web server socket, %s", err)
		}

		log.Infof("Web server listening on unix socket %s.", path)
	}

	// Configure server
	server.Handler = router
}

func runWebServer() {
	err := server.Serve(listener)
	if err == http.ErrServerClosed {
		log.Info("Web server closed.")
	} else {
		log.Errorf("Error starting server, %s", err)
	}
}

func registerWebpage() {
	router.GET("/", func(context *gin.Context) {
		context.Redirect(http.StatusTemporaryRedirect, "web")
	})

	if stat, err := os.Stat("assets/public"); err == nil && stat.IsDir() {
		log.Info("Serving web interface from filesystem.")
		router.Static("/web", "assets/public")
	} else {
		log.Info("Serving bundled assets.")
		var public fs.FS
		public, err = fs.Sub(assets, "assets/public")
		if err != nil {
			log.Fatalf("Error getting subdirectory, %s", err)
		}
		router.StaticFS("/web", http.FS(public))
	}
}
